@@ -10,9 +10,3 @@
 
 #endif // BESCHI_IMPLEMENTATION
 
-#ifdef __cplusplus
-}
-#endif
-
-#endif // INCLUDE_BESCHI_H
-
