@@ -1,12 +1,13 @@
-package Beschi
+package beschi
 
 import (
 	"encoding/binary"
+	"fmt"
 	"io"
 )
 
 func readString(data io.Reader, str *string) error {
-	var len uint32
+	var len {# STRING_SIZE_TYPE #}
 	binary.Read(data, binary.LittleEndian, &len)
 	sbytes := make([]byte, len)
 	err := binary.Read(data, binary.LittleEndian, &sbytes)
@@ -18,7 +19,15 @@ func readString(data io.Reader, str *string) error {
 }
 
 func writeString(data io.Writer, str *string) {
-	strLen := (uint32)(len(*str))
+	strLen := ({# STRING_SIZE_TYPE #})(len(*str))
 	binary.Write(data, binary.LittleEndian, strLen)
 	io.WriteString(data, *str)
 }
+
+func getDataOffset(data io.Reader) int64 {
+	if seeker, ok := data.(io.Seeker); ok {
+		offset, _ := seeker.Seek(0, io.SeekCurrent)
+		return offset
+	}
+	return -1
+}
